-- Escape Frustration
-- By: abug (aka Ienen) warcraftbug2@gmail.com

local StaticPopupDialogs = StaticPopupDialogs

EscapeFrustrationData = {
    CheckButtonChoices = {
        RESURRECT = false,  -- include with this one: RESURRECT_NO_SICKNESS, RESURRECT_NO_TIMER
        PARTY_INVITE = false,
        GUILD_INVITE = false,
        ARENA_TEAM_INVITE = false,
        QUEST_ACCEPT = false,  -- include with this one: QUEST_ACCEPT_LOG_FULL (?)
        AREA_SPIRIT_HEAL = false,
        CONFIRM_SUMMON = false,
    },
}

local checkButtons = { }
local checkButtonChoices
local optionsPanel

local function eventHandler(self)
    self:UnregisterEvent("VARIABLES_LOADED")
    StaticPopupDialogs = _G.StaticPopupDialogs
    checkButtonChoices = EscapeFrustrationData.CheckButtonChoices
    for _,checkButton in pairs(checkButtons) do
        -- click twice to initialize the checkbutton
        checkButton:Click()
        checkButton:Click()
    end
end

local function checkButtonHandler(self)
    local choice = self.name
    local otherPopupsTable = self.otherPopupsTable
    
    if checkButtonChoices[choice] then
        -- make the check button checked
        self:SetChecked(true)
        checkButtonChoices[choice] = false
        StaticPopupDialogs[choice].hideOnEscape = nil
        if otherPopupsTable then
            for _,popupName in pairs(otherPopupsTable) do
                StaticPopupDialogs[popupName].hideOnEscape = nil
            end
        end
    else
        -- make the check button unchecked
        self:SetChecked(false)
        checkButtonChoices[choice] = true
        StaticPopupDialogs[choice].hideOnEscape = 1
        if otherPopupsTable then
            for i,popupName in pairs(otherPopupsTable) do
                StaticPopupDialogs[popupName].hideOnEscape = 1
            end
        end
    end
end

local function generateCheckButton(name, text, otherPopupsTable, previousCheckButton)
    local newCheckButton = CreateFrame("CheckButton", nil, optionsPanel, "UICheckButtonTemplate")
    newCheckButton:ClearAllPoints()
    newCheckButton:SetScript("OnClick", checkButtonHandler)
    newCheckButton:SetHeight(25)
    newCheckButton:SetWidth(25)
    if previousCheckButton then
        newCheckButton:SetPoint("TOPLEFT", previousCheckButton, "BOTTOMLEFT")
    end
    
    local newLabel = newCheckButton:CreateFontString()
    newLabel:ClearAllPoints()
    newLabel:SetPoint("TOPLEFT", newCheckButton, "TOPRIGHT", -8)
    newLabel:SetHeight(25)
    newLabel:SetWidth(300)
    newLabel:SetFontObject("GameFontNormal")
    newLabel:SetJustifyH("LEFT")
    newLabel:SetText(text)
    
    newCheckButton.label = newLabel
    newCheckButton.otherPopupsTable = otherPopupsTable
    newCheckButton.name = name
    
    checkButtons[name] = newCheckButton
    
    return newCheckButton
end

optionsPanel = CreateFrame("Frame", "EscapeFrustrationoptionsPanel")
optionsPanel:SetBackdrop({
    bgFile = "Interface/DialogFrame/UI-DialogBox-Background", 
    edgeFile = "Interface/DialogFrame/UI-DialogBox-Border", 
    tile = true,
    tileSize = 8,
    edgeSize = 8,
    insets = {
        left = 4,
        right = 4,
        top = 4,
        bottom = 4
    },
})
optionsPanel:SetWidth(385)
optionsPanel:SetHeight(409)

optionsPanel:RegisterEvent("VARIABLES_LOADED")
optionsPanel:SetScript("OnEvent", eventHandler)

local descriptionFontString = optionsPanel:CreateFontString()
descriptionFontString:ClearAllPoints()
descriptionFontString:SetPoint("TOPLEFT", optionsPanel, "TOPLEFT", 8, -3)
descriptionFontString:SetPoint("TOPRIGHT", optionsPanel, "TOPRIGHT", -8, -3)
descriptionFontString:SetHeight(50)
descriptionFontString:SetFontObject("GameFontNormal")
descriptionFontString:SetText("The following checkboxes change the |cffffffffEscape|r key cancelling behavior of several pop-ups.\n(A |TInterface\\Buttons\\UI-CheckBox-Check:16:16:0:12|t indicates that collection of pop-ups will |cffff0000ignore|r |cffffffffEscape|r presses.)")

local selectAllButton = CreateFrame("Button", "EscapeFrustration_SelectAllButton", optionsPanel, "UIPanelButtonTemplate")
selectAllButton:ClearAllPoints()
selectAllButton:SetPoint("TOPLEFT", descriptionFontString, "BOTTOMLEFT", 8)
selectAllButton:SetHeight(25)
selectAllButton:SetWidth(100)
selectAllButton:SetText("Select All")
selectAllButton:SetScript("OnClick", function(self)
    for name,checkButton in pairs(checkButtons) do
        checkButtonChoices[name] = true
        checkButton:Click()
    end
end)
local selectNoneButton = CreateFrame("Button", "EscapeFrustration_SelectNoneButton", optionsPanel, "UIPanelButtonTemplate")
selectNoneButton:ClearAllPoints()
selectNoneButton:SetPoint("TOPLEFT", selectAllButton, "TOPRIGHT", 8)
selectNoneButton:SetHeight(25)
selectNoneButton:SetWidth(100)
selectNoneButton:SetText("Select None")
selectNoneButton:SetScript("OnClick", function(self)
    for name,checkButton in pairs(checkButtons) do
        checkButtonChoices[name] = false
        checkButton:Click()
    end
end)

-- create check buttons
local previousCheckButton = generateCheckButton("RESURRECT", "Resurrect Dialogs", {"RESURRECT_NO_SICKNESS", "RESURRECT_NO_TIMER"}, selectAllButton)
previousCheckButton = generateCheckButton("PARTY_INVITE", "Party / Raid Invites", nil, previousCheckButton)
previousCheckButton = generateCheckButton("GUILD_INVITE", "Guild Invites", nil, previousCheckButton)
previousCheckButton = generateCheckButton("ARENA_TEAM_INVITE", "Arena Team Invites", nil, previousCheckButton)
previousCheckButton = generateCheckButton("QUEST_ACCEPT", "Quest Dialog", {"QUEST_ACCEPT_LOG_FULL"}, previousCheckButton)
previousCheckButton = generateCheckButton("AREA_SPIRIT_HEAL", "Battleground Spirit Guides", nil, previousCheckButton)
previousCheckButton = generateCheckButton("CONFIRM_SUMMON", "Summon Dialog", nil, previousCheckButton)

-- Adding optionsPanel to the Interface Options
optionsPanel.name = "Escape Frustration"
--optionaPanel.okay = function(self) end
--optionaPanel.cancel = function(self) end
--optionaPanel.default = function(self) end
InterfaceOptions_AddCategory(optionsPanel)